import numpy as np
import tensorflow as tf
import pandas as pd

INPUT_SIZE = 8
HIDDEN_SIZE = 5
OUTPUT_SIZE = 1

def average_weights(file_paths):
    # Initialize accumulators for weights and biases
    weights_input_hidden_sum = np.zeros((INPUT_SIZE, HIDDEN_SIZE))
    weights_hidden_output_sum = np.zeros((HIDDEN_SIZE, OUTPUT_SIZE))
    bias_hidden_sum = np.zeros(HIDDEN_SIZE)
    bias_output_sum = np.zeros(OUTPUT_SIZE)
    num_files = len(file_paths)
    
    # Accumulate weights from each file
    for file_path in file_paths:
        try:
            data = np.fromfile(file_path, dtype=np.float64)
            if data.size != (INPUT_SIZE * HIDDEN_SIZE) + (HIDDEN_SIZE * OUTPUT_SIZE) + HIDDEN_SIZE + OUTPUT_SIZE:
                raise ValueError(f"Unexpected size in file: {file_path}")

            # Accumulate weights and biases from the data
            weights_input_hidden_sum += data[:INPUT_SIZE * HIDDEN_SIZE].reshape(INPUT_SIZE, HIDDEN_SIZE)
            weights_hidden_output_sum += data[INPUT_SIZE * HIDDEN_SIZE: INPUT_SIZE * HIDDEN_SIZE + HIDDEN_SIZE * OUTPUT_SIZE].reshape(HIDDEN_SIZE, OUTPUT_SIZE)
            bias_hidden_sum += data[INPUT_SIZE * HIDDEN_SIZE + HIDDEN_SIZE * OUTPUT_SIZE: INPUT_SIZE * HIDDEN_SIZE + HIDDEN_SIZE * OUTPUT_SIZE + HIDDEN_SIZE]
            bias_output_sum += data[-OUTPUT_SIZE:]

        except Exception as e:
            print(f"Error processing file {file_path}: {e}")
    
    # Calculate average weights and biases
    weights_input_hidden_avg = weights_input_hidden_sum / num_files
    weights_hidden_output_avg = weights_hidden_output_sum / num_files
    bias_hidden_avg = bias_hidden_sum / num_files
    bias_output_avg = bias_output_sum / num_files

    # Save averaged weights to a single file
    with open('averaged_weights.bin', 'wb') as f:
        f.write(weights_input_hidden_avg.astype(np.float64).tobytes())
        f.write(weights_hidden_output_avg.astype(np.float64).tobytes())
        f.write(bias_hidden_avg.astype(np.float64).tobytes())
        f.write(bias_output_avg.astype(np.float64).tobytes())

def load_weights(file_path):
    # Load averaged weights from the file
    with open(file_path, 'rb') as f:
        data = np.frombuffer(f.read(), dtype=np.float64)
        
        weights_input_hidden = data[:INPUT_SIZE * HIDDEN_SIZE].reshape(INPUT_SIZE, HIDDEN_SIZE)
        weights_hidden_output = data[INPUT_SIZE * HIDDEN_SIZE: INPUT_SIZE * HIDDEN_SIZE + HIDDEN_SIZE * OUTPUT_SIZE].reshape(HIDDEN_SIZE, OUTPUT_SIZE)
        bias_hidden = data[INPUT_SIZE * HIDDEN_SIZE + HIDDEN_SIZE * OUTPUT_SIZE: INPUT_SIZE * HIDDEN_SIZE + HIDDEN_SIZE * OUTPUT_SIZE + HIDDEN_SIZE]
        bias_output = data[-OUTPUT_SIZE:]
    
    return weights_input_hidden, weights_hidden_output, bias_hidden, bias_output

def create_model(weights_input_hidden, weights_hidden_output, bias_hidden, bias_output):
    # Create a model with the same architecture as in C++ code
    model = tf.keras.Sequential([
        tf.keras.layers.Dense(HIDDEN_SIZE, activation='sigmoid', input_shape=(INPUT_SIZE,), use_bias=True),
        tf.keras.layers.Dense(OUTPUT_SIZE, activation='sigmoid', use_bias=True)
    ])
    
    # Set weights for the model
    model.layers[0].set_weights([weights_input_hidden, bias_hidden])
    model.layers[1].set_weights([weights_hidden_output, bias_output])
    
    return model

# Paths to weight files
file_paths = [
    'weights_node01.bin',
    'weights_node02.bin',
    'weights_node03.bin',
    'weights_node04.bin',
    'weights_node05.bin'
]

# Average weights and create a weights file
average_weights(file_paths)

# Load averaged weights
weights_input_hidden, weights_hidden_output, bias_hidden, bias_output = load_weights('averaged_weights.bin')

# Create and compile the model
model = create_model(weights_input_hidden, weights_hidden_output, bias_hidden, bias_output)
model.compile(optimizer='adam', loss='binary_crossentropy', metrics=['accuracy'])

# Load the diabetes dataset
data = pd.read_csv('diabetes.csv')
X = data.iloc[:, :-1].values
y = data.iloc[:, -1].values

# Train the model
model.fit(X, y, epochs=10, batch_size=32)

# Make predictions
input_data = np.array([[5, 140, 80, 30, 120, 35.0, 0.550, 45]])

predictions = model.predict(input_data)
print("Predictions:", predictions)

binary_predictions = (predictions > 0.5).astype(int)
print("Binary Predictions:", binary_predictions)

